//	by EvilJess aka Jeffrey Bakker 2004

//	New or modified code (C)2004 Jeffrey Bakker <evil@damnclan.com>
//	Existing code for Unreal  Tournament 2003 Copyright  2002 Epic Games, Inc.
//	Existing code for Unreal  Tournament 2004 Copyright  2004 Epic Games, Inc.


//	BinStaGib - InstaGib meets BinSlayer
//	Copyright (C) 2004  Jeffrey Bakker

//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.

//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.

//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class DeathFire extends SuperShockBeamFire;

var class<DamageType> DamageTypeHead;
var class<DamageType> DamageTypeSplash;
var float SplashDamage;

var bool bHurtEntry;

function bool AllowMultiHit()
{
	return true;
}

simulated function DelayedHurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;
	
	if( bHurtEntry )
		return;

	bHurtEntry = true;
	foreach Weapon.VisibleCollidingActors( class'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Victims.Role == ROLE_Authority) && (!Victims.IsA('FluidSurfaceInfo')) )
		{
			if(Pawn(Victims) != Instigator)
			{
				dir = Victims.Location - HitLocation;
				dist = FMax(1,VSize(dir));
				dir = dir/dist; 
				damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
				if ( DamageType.Default.bDelayedDamage && (Instigator == None) && (Pawn(Victims) != None) )
					Pawn(Victims).DelayedDamageInstigatorController = Instigator.Controller;
				Victims.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator, 
					Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType
				);
//				if (Vehicle(Victims) != None && Vehicle(Victims).Health > 0)
//					Vehicle(Victims).DriverRadiusDamage(DamageAmount, DamageRadius, InstigatorController, DamageType, Momentum, HitLocation);
			}
		} 
	}
	bHurtEntry = false;
}

function TracePart(Vector Start, Vector End, Vector X, Rotator Dir, Pawn Ignored)
{
	local Vector HitLocation, HitNormal;
	local Actor Other;
	local float dist;

	Other = Ignored.Trace(HitLocation, HitNormal, End, Start, true);

	if ( (Other != None) && (Other != Ignored) )
	{
		if ( !Other.bWorldGeometry )
		{

			if(	(Pawn(Other) != None) &&
				(Other.GetClosestBone(HitLocation,X,dist,'head',10) == 'head')	)
			{
				//Pawn(Other).Died(Instigator.Controller, DamageTypeHead, Location);
				Other.TakeDamage(DamageMax, Instigator, HitLocation, 1*X, DamageTypeHead);
			} else {

				Other.TakeDamage(DamageMax, Instigator, HitLocation, Momentum*X, DamageType);

				if(DamageType == class'BinStaGib04.DamTypeDeathStorm')
					Instigator.ReceiveLocalizedMessage(class'BlownAway', 0, Instigator.PlayerReplicationInfo);
			}
			HitNormal = Vect(0,0,0);
			if ( (Pawn(Other) != None) && (HitLocation != Start) && AllowMultiHit() )
			{
				TracePart(HitLocation,End,X,Dir,Pawn(Other));
			}
		}
	}
	else
	{
		HitLocation = End;
 		HitNormal = Vect(0,0,0);
	}
	SpawnBeamEffect(Start, Dir, HitLocation, HitNormal, 0);
	DelayedHurtRadius(SplashDamage, 124, DamageTypeSplash, Momentum * 0.25, HitLocation );
//	PlaySound(Sound'WeaponSounds.BaseImpactAndExplosions.BLightningGunExplosion');
}

function SpawnBeamEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal, int ReflectNum)
{
	local ShockBeamEffect Beam;
    
	if ( (Instigator.PlayerReplicationInfo.Team != None) && (Instigator.PlayerReplicationInfo.Team.TeamIndex == 1) )
		Beam = Spawn(class'BinStaGib04.DeathBeamBlueEffect',,, Start, Dir);
	else
		Beam = Spawn(BeamEffectClass,,, Start, Dir);
	if (ReflectNum != 0) Beam.Instigator = None; // prevents client side repositioning of beam start
	Beam.AimAt(HitLocation, HitNormal);
}

function StartBerserk()
{
	DamageType = class'BinStaGib04.DamTypeDeathStorm';
	FireRate *= 0.500000;

//	DamageMin = 66;
//	DamageMax = 66;
}

function StopBerserk()
{
	DamageType = default.DamageType;
	FireRate = default.FireRate;

//	DamageMin = default.DamageMin;
//	DamageMax = default.DamageMax;
}

defaultproperties
{
	bHurtEntry=False

	SplashDamage=84.000000
	BeamEffectClass=Class'BinStaGib04.DeathBeamEffect'
	DamageType=Class'BinStaGib04.DamTypeInstaDeath'
	DamageTypeHead=Class'BinStaGib04.DamTypeUberDeath'
	DamageTypeSplash=Class'BinStaGib04.DamTypeDeathSplash'
	DamageMin=666
 	Momentum=100000.000000
//	FireSound=Sound'WeaponSounds.BaseGunTech.BLockOn1'
	FireRate=0.481300
	AmmoClass=Class'BinStaGib04.DeathAmmo'
	AmmoPerFire=0
}
